@@ -2,7 +2,7 @@
 #define _CEGOXS_H_INCLUDED   1
 
 #define NEED_DBIXS_VERSION 93
-#include <sys/vnode.h>
+/* #include <sys/vnode.h> */
 #include <sys/stat.h>
 
 #include <DBIXS.h>
