@@ -88,7 +88,7 @@ cego_db_login(SV *dbh, imp_dbh_t *imp_dbh, char *dbname, char *user, char *pass)
 	return FALSE;
     }
 
-    imp_dbh->cgnet = new CegoNet( protocol, logFile, logMode );
+    imp_dbh->cgnet = new CegoNet( protocol, logFile, Chain(""), logMode );
     
     try
     {
@@ -293,7 +293,6 @@ cego_st_prepare (SV *sth, imp_sth_t *imp_sth,
                 char *statement, SV *attribs)
 {
 
-
     dTHR;
     D_imp_dbh_from_sth;
 
@@ -302,10 +301,21 @@ cego_st_prepare (SV *sth, imp_sth_t *imp_sth,
     DBIc_IMPSET_on(imp_sth);
 
     int numParams=0;
-    Chain c = Chain(statement);
-    if ( c.cutTrailing(" ").subChain(1,1) == Chain("?") )
+    Chain c = Chain(statement).cutTrailing(" ");
+
+    if ( c.subChain(1,1) == Chain("?") )
+    {
+	imp_sth->hasRetValue=true;
 	numParams++;
-	
+    }
+    else
+    {
+	imp_sth->hasRetValue=false;
+    }
+
+    if ( c.subChain(c.length()-1,c.length()-1) == Chain("?") )
+	numParams++;
+
     Tokenizer tok(Chain(statement), "?");
     imp_sth->stmtChunks = new ListT<Chain>;
 
@@ -342,7 +352,6 @@ cego_st_execute (SV *sth, imp_sth_t *imp_sth)
     I32 i;
     int retval;
 
-
     if ( imp_dbh->cgnet == NULL )
     {
 	Chain msg = Chain("Invalid database handle");
@@ -364,11 +373,11 @@ cego_st_execute (SV *sth, imp_sth_t *imp_sth)
     else
     {
 
-	int pos=1;
+	pos=1;
 
 	Chain* pChunk = imp_sth->stmtChunks->First();
 
-	if ( num_params == imp_sth->stmtChunks->Size() )
+	if ( imp_sth->hasRetValue )
 	{
 	    Chain id(pos);
 	    CegoDBDParam* pParam = imp_sth->paramList->Find( CegoDBDParam(id) );
@@ -412,12 +421,24 @@ cego_st_execute (SV *sth, imp_sth_t *imp_sth)
 
 	    pos++;
 	}
+	
+	
+
+	// find trailing parameter 
+	Chain id(pos);
+	CegoDBDParam* pParam = imp_sth->paramList->Find( CegoDBDParam(id) );
+	if ( pParam )
+	{
+	    if ( pParam->getRef() == 0 )
+		stmt += pParam->getValue(); 
+	    else
+		stmt += Chain(":p") + pParam->getId();	       
+	}
     }
-
-
+    
     // normalize to plain statement
     stmt = stmt.cutTrailing(" ;");
-   
+    
     if (stmt == Chain("quit") )
 	return 0;
     
@@ -520,7 +541,10 @@ cego_bind_ph (SV *sth, imp_sth_t *imp_sth,
     }
     else if ( sql_type == SQL_VARCHAR ) 
     {
-	qVal = Chain("'") + Chain(pval) + Chain("'");
+	Chain rawVal(pval);
+	Chain escVal;
+	rawVal.replaceAll(Chain("'"), Chain("''"), escVal);
+	qVal = Chain("'") + Chain(escVal) + Chain("'");
     }
     else
     {
@@ -556,7 +580,6 @@ cego_bind_ph (SV *sth, imp_sth_t *imp_sth,
 	}
     }
     return TRUE;
-
 }
 
 AV *
